@@ -76,6 +76,8 @@ def arrange_by_flags(self):
 
     flagTypes = {}
     for r in self.rules:
+        if not r.flags:
+            r.flags = 0
         if not r.flags in flagTypes:
             flagTypes[r.flags] = []
         flagTypes[r.flags].append(r)
@@ -143,54 +145,53 @@ def feaPreamble(self, ff):
     preamble = []
     for r in self.rules:
         preamble.extend(r.feaPreamble(ff))
-    if hasattr(self, "markFilteringSet"):
+    if self.flags & 0xFF00:
+        assert(self.markAttachmentSet is not None)
+        self.markAttachmentSetAsClass = ff.getNamedClassFor(self.markAttachmentSet, gensym("markAttachmentSet"))
+    if self.flags & 0x10:
+        assert(self.markFilteringSet is not None)
         self.markFilteringSetAsClass = ff.getNamedClassFor(self.markFilteringSet, gensym("markFilteringSet"))
     return preamble
 
 
-def asFeaAST(self):
-    if self.name:
+def asFeaAST(self, inFeature=False):
+    if self.name and not inFeature:
+        f = feaast.LookupBlock(name=self.name)
+    elif self.name:
         f = feaast.LookupBlock(name=self.name)
     else:
         f = feaast.Block()
-
     arranged = arrange(self)
 
-    if arranged:
+    if arranged and inFeature:
+        f = feaast.Block()
         for a in arranged:
-            f.statements.append(asFeaAST(a))
+            f.statements.append(asFeaAST(a, inFeature))
         return f
 
-    if self.languages and not (
-        self.languages[0][0] == "DFLT" and self.languages[0][1] == "dflt"
-    ):
-        s, l = self.languages[0]
-        f.statements.append(feaast.ScriptStatement(s))
-        if l != "*":
-            l = "%4s" % l
-            f.statements.append(feaast.LanguageStatement(l))
-
     if hasattr(self, "flags") and self.flags > 0:
-        if self.flags & 0xFF00:
-            # This is dirty and wrong and I feel bad.
-            flags = feaast.LookupFlagStatement(
-                self.flags,
-                markAttachment=feaast.Comment("@MarkClass%i" % (self.flags >> 8)),
-            )
-        else:
-            flags = feaast.LookupFlagStatement(self.flags)
-        if hasattr(self, "markFilteringSetAsClass"):
+        flags = feaast.LookupFlagStatement(self.flags)
+        if self.flags & 0x10 and hasattr(self, "markFilteringSetAsClass"): # XXX
             # We only need the name, not the contents
             mfs = feaast.GlyphClassDefinition(self.markFilteringSetAsClass,
                 feaast.GlyphClass([])
             )
             flags.markFilteringSet = feaast.GlyphClassName(mfs)
+        if self.flags & 0xFF00 and hasattr(self, "markAttachmentSetAsClass"): # XXX
+            mfs = feaast.GlyphClassDefinition(self.markAttachmentSetAsClass,
+                feaast.GlyphClass([])
+            )
+            flags.markAttachment=feaast.GlyphClassName(mfs)
+
         f.statements.append(flags)
 
     for x in self.comments:
-        f.statements.append(Comment(x))
+        f.statements.append(feaast.Comment(x))
 
-    lastaddress = None
+    f.statements.append(feaast.Comment(";"))
+    lastaddress = self.address
+    if lastaddress:
+        f.statements.append(feaast.Comment("# Original source: %s " % (" ".join([str(x) for x in lastaddress]))))
     for x in self.rules:
         if x.address and x.address != lastaddress:
             f.statements.append(feaast.Comment("# Original source: %s " % x.address))
